/****************************************************************************** */
/* usbtick.c - Functions related to USB stack tick timer handling. */
/****************************************************************************** */
/**
 *  Copyright (c) Texas Instruments Incorporated 2015-2019
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/


#include "types.h"
#include "debug.h"
#include "usblib.h"
#include "usblibpriv.h"

/****************************************************************************** */
/*! \addtogroup general_usblib_api */
/*! @{ */
/****************************************************************************** */

/****************************************************************************** */
/* Define a boolean type, and values for true and false. */
/****************************************************************************** */
#ifndef true
#define true 1
#endif

#ifndef false
#define false 0
#endif

#ifndef NULL
#define NULL ((void*) 0)
#endif

/****************************************************************************** */
/* These are the internal timer tick handlers used by the USB stack.  Handlers */
/* in g_pfTickHandlers are called in the context of the USB SOF interrupt */
/* every USB_SOF_TICK_DIVIDE milliseconds. */
/****************************************************************************** */
tUSBTickHandler g_pfTickHandlers[USB_TICK_HANDLER_NUM];
void *g_pvTickInstance[USB_TICK_HANDLER_NUM];

/****************************************************************************** */
/* Flag to indicate whether or not we have been initialized. */
/****************************************************************************** */
tBoolean g_bUSBTimerInitialized = false;

/****************************************************************************** */
/* This is the current tick value in ms for the system.  This is used for all */
/* instances of USB controllers and for all timer tick handlers. */
/****************************************************************************** */
uint32_t g_ulCurrentUSBTick = 0;

/****************************************************************************** */
/* This is the total number of SOF interrupts received since the system */
/* booted.  The value is incremented by the low level device- or host-interrupt */
/* handler functions. */
/****************************************************************************** */
uint32_t g_ulUSBSOFCount = 0;

/****************************************************************************** */
/* This internal function initializes the variables used in processing timer */
/* ticks. */
/* This function should only be called from within the USB library.  It is set */
/* up to ensure that it can be called multiple times if necessary without */
/* the previous configuration being erased (to cater for OTG mode switching). */
/* \return None. */
/****************************************************************************** */
void
InternalUSBTickInit(void)
{
    uint32_t ulLoop;

    if(!g_bUSBTimerInitialized)
    {
        for(ulLoop = 0; ulLoop < USB_TICK_HANDLER_NUM; ulLoop++)
        {
            g_pfTickHandlers[ulLoop] = (tUSBTickHandler)0;
            g_pvTickInstance[ulLoop] = 0;
        }

        g_bUSBTimerInitialized = true;
    }
}

/****************************************************************************** */
/* This internal function handles registering OTG, Host, or Device SOF timer */
/* handler functions. */
/* \param ulHandler specifies which type of handler to register. */
/* \param pfHandler specifies the handler to call for the given type of */
/* handler. */
/* This function should only be called inside the USB library and only as a */
/* result to a call to reinitialize the stack in a new mode.  Currently the */
/* following 3 types of timer tick handlers can be registered: */
/* TICK_HANDLER_OTG, TICK_HANDLER_HOST, or TICK_HANDLER_DEVICE.  Handlers */
/* registered via this function are called in the context of the SOF interrupt. */
/* \return None. */
/****************************************************************************** */
void
InternalUSBRegisterTickHandler(uint32_t ulHandler,
                               tUSBTickHandler pfHandler,
                               void *pvInstance)
{
    ASSERT(ulHandler < USB_TICK_HANDLER_NUM);

    /* Save the handler. */
    g_pfTickHandlers[ulHandler] = pfHandler;

    /* Save the instance data. */
    g_pvTickInstance[ulHandler] = pvInstance;
}

/****************************************************************************** */
/*! \internal */
/*! */
/*! Calls internal handlers in response to a tick based on the start of frame */
/*! interrupt. */
/*! */
/*! \param ulTicksmS specifies how many milliseconds have passed since the last */
/*! call to this function. */
/*! */
/*! This function is called every 5mS in the context of the Start of Frame */
/*! (SOF) interrupt.  It is used to call any registered internal tick */
/*! functions. */
/*! */
/*! This function should only be called from within the USB library. */
/*! */
/*! \return None. */
/****************************************************************************** */
void InternalUSBStartOfFrameTick(uint32_t ulTicksmS, 
                                 uint32_t ulIndex)
{
    int iIdx;

    /* Advance time. */
    g_ulCurrentUSBTick += ulTicksmS;

    /* Call any registered SOF tick handlers. */
    for(iIdx = 0; iIdx < USB_TICK_HANDLER_NUM; iIdx++)
    {
        if(g_pfTickHandlers[iIdx])
        {
            g_pfTickHandlers[iIdx](g_pvTickInstance[iIdx], ulTicksmS, ulIndex);
        }
    }
}

/****************************************************************************** */
/* Close the Doxygen group. */
/*! @} */
/****************************************************************************** */
